@@ -1,5 +1,6 @@
 use dashmap::DashMap;
 use serde::{Deserialize, Serialize};
+use serde_json::Value;
 use tower_lsp::jsonrpc::Result;
 use tower_lsp::lsp_types::*;
 use tower_lsp::{Client, LanguageServer};
@@ -9,6 +10,7 @@ use ast_grep_config::{CombinedScan, RuleCollection, RuleConfig};
 use ast_grep_core::{language::Language, AstGrep, Doc, Node, NodeMatch, StrDoc};
 
 use std::collections::HashMap;
+use std::path::PathBuf;
 
 pub use tower_lsp::{LspService, Server};
 
@@ -67,6 +69,11 @@ impl<L: LSPLang> Backend<L> {
 
 const FALLBACK_CODE_ACTION_PROVIDER: Option<CodeActionProviderCapability> =
   Some(CodeActionProviderCapability::Simple(true));
+
+const SOURCE_FIX_ALL_AST_GREP: CodeActionKind = CodeActionKind::new("source.fixAll.ast-grep");
+
+pub const APPLY_ALL_FIXES: &str = "ast-grep.applyAllFixes";
+
 fn code_action_provider(
   client_capability: &ClientCapabilities,
 ) -> Option<CodeActionProviderCapability> {
@@ -81,7 +88,11 @@ fn code_action_provider(
     return None;
   }
   Some(CodeActionProviderCapability::Options(CodeActionOptions {
-    code_action_kinds: Some(vec![CodeActionKind::QUICKFIX]),
+    code_action_kinds: Some(vec![
+      CodeActionKind::QUICKFIX,
+      CodeActionKind::SOURCE_FIX_ALL,
+      SOURCE_FIX_ALL_AST_GREP,
+    ]),
     work_done_progress_options: Default::default(),
     resolve_provider: Some(true),
   }))
@@ -100,6 +111,10 @@ impl<L: LSPLang> LanguageServer for Backend<L> {
         text_document_sync: Some(TextDocumentSyncCapability::Kind(TextDocumentSyncKind::FULL)),
         code_action_provider: code_action_provider(&params.capabilities)
           .or(FALLBACK_CODE_ACTION_PROVIDER),
+        execute_command_provider: Some(ExecuteCommandOptions {
+          commands: vec![APPLY_ALL_FIXES.to_string()],
+          work_done_progress_options: Default::default(),
+        }),
         ..ServerCapabilities::default()
       },
     })
@@ -184,12 +199,12 @@ impl<L: LSPLang> LanguageServer for Backend<L> {
   }
 
   async fn code_action(&self, params: CodeActionParams) -> Result<Option<CodeActionResponse>> {
-    self
-      .client
-      .log_message(MessageType::INFO, "run code action!")
-      .await;
     Ok(self.on_code_action(params).await)
   }
+
+  async fn execute_command(&self, params: ExecuteCommandParams) -> Result<Option<Value>> {
+    Ok(self.on_execute_command(params).await)
+  }
 }
 
 fn convert_node_to_range<D: Doc>(node_match: &Node<D>) -> Range {
@@ -266,6 +281,19 @@ fn url_to_code_description(url: &Option<String>) -> Option<CodeDescription> {
   Some(CodeDescription { href })
 }
 
+fn build_error_id_to_ranges(diagnostics: Vec<Diagnostic>) -> HashMap<String, Vec<Range>> {
+  let mut error_id_to_ranges = HashMap::new();
+  for diagnostic in diagnostics {
+    let rule_id = match diagnostic.code {
+      Some(NumberOrString::String(rule)) => rule,
+      _ => continue,
+    };
+    let ranges = error_id_to_ranges.entry(rule_id).or_insert_with(Vec::new);
+    ranges.push(diagnostic.range);
+  }
+  error_id_to_ranges
+}
+
 impl<L: LSPLang> Backend<L> {
   pub fn new(client: Client, rules: std::result::Result<RuleCollection<L>, String>) -> Self {
     Self {
@@ -274,30 +302,41 @@ impl<L: LSPLang> Backend<L> {
       map: DashMap::new(),
     }
   }
-  async fn publish_diagnostics(&self, uri: Url, versioned: &VersionedAst<StrDoc<L>>) -> Option<()> {
+  fn get_diagnostics(
+    &self,
+    uri: &Url,
+    versioned: &VersionedAst<StrDoc<L>>,
+  ) -> Option<Vec<Diagnostic>> {
     let mut diagnostics = vec![];
     let path = uri.to_file_path().ok()?;
 
     let rules = match &self.rules {
       Ok(rules) => rules.for_path(&path),
       Err(_) => {
-        return Some(());
+        return Some(diagnostics);
       }
     };
+
     let scan = CombinedScan::new(rules);
     let hit_set = scan.all_kinds();
     let matches = scan.scan(&versioned.root, hit_set, false).matches;
     for (id, ms) in matches {
       let rule = scan.get_rule(id);
-      let to_diagnostic = |m| convert_match_to_diagnostic(m, rule, &uri);
+      let to_diagnostic = |m| convert_match_to_diagnostic(m, rule, uri);
       diagnostics.extend(ms.into_iter().map(to_diagnostic));
     }
+    Some(diagnostics)
+  }
+
+  async fn publish_diagnostics(&self, uri: Url, versioned: &VersionedAst<StrDoc<L>>) -> Option<()> {
+    let diagnostics = self.get_diagnostics(&uri, versioned).unwrap_or_default();
     self
       .client
       .publish_diagnostics(uri, diagnostics, Some(versioned.version))
       .await;
     Some(())
   }
+
   async fn on_open(&self, params: DidOpenTextDocumentParams) -> Option<()> {
     let text_doc = params.text_document;
     let uri = text_doc.uri.as_str().to_owned();
@@ -350,25 +389,31 @@ impl<L: LSPLang> Backend<L> {
     self.map.remove(params.text_document.uri.as_str());
   }
 
-  async fn on_code_action(&self, params: CodeActionParams) -> Option<CodeActionResponse> {
-    let text_doc = params.text_document;
-    let uri = text_doc.uri.as_str();
-    let path = text_doc.uri.to_file_path().ok()?;
-    let diagnostics = params.context.diagnostics;
-    let error_id_to_ranges = Self::build_error_id_to_ranges(diagnostics);
+  fn compute_all_fixes(
+    &self,
+    text_document: TextDocumentIdentifier,
+    error_id_to_ranges: HashMap<String, Vec<Range>>,
+    path: PathBuf,
+  ) -> Option<HashMap<Url, Vec<TextEdit>>>
+  where
+    L: ast_grep_core::Language + std::cmp::Eq,
+  {
+    let uri = text_document.uri.as_str();
     let versioned = self.map.get(uri)?;
-    let mut response = CodeActionResponse::new();
-    let rules = if let Ok(rules) = &self.rules {
-      rules
-    } else {
-      return Some(response);
+    let mut changes: HashMap<Url, Vec<TextEdit>> = HashMap::new();
+    let edits = changes.entry(text_document.uri.clone()).or_default();
+
+    let Ok(rules) = &self.rules else {
+      return None;
     };
+
     for config in rules.for_path(&path) {
       let ranges = match error_id_to_ranges.get(&config.id) {
         Some(ranges) => ranges,
         None => continue,
       };
       let matcher = &config.matcher;
+
       for matched_node in versioned.root.root().find_all(&matcher) {
         let range = convert_node_to_range(&matched_node);
         if !ranges.contains(&range) {
@@ -383,40 +428,60 @@ impl<L: LSPLang> Backend<L> {
           range,
           new_text: String::from_utf8(edit.inserted_text).unwrap(),
         };
-        let mut changes = HashMap::new();
-        changes.insert(text_doc.uri.clone(), vec![edit]);
-        let edit = Some(WorkspaceEdit {
-          changes: Some(changes),
-          document_changes: None,
-          change_annotations: None,
-        });
-        let action = CodeAction {
-          title: config.message.clone(),
-          command: None,
-          diagnostics: None,
-          edit,
-          disabled: None,
-          kind: Some(CodeActionKind::QUICKFIX),
-          is_preferred: Some(true),
-          data: None,
-        };
-        response.push(CodeActionOrCommand::from(action));
+
+        edits.push(edit);
       }
     }
-    Some(response)
+    Some(changes)
   }
 
-  fn build_error_id_to_ranges(diagnostics: Vec<Diagnostic>) -> HashMap<String, Vec<Range>> {
-    let mut error_id_to_ranges = HashMap::new();
-    for diagnostic in diagnostics {
-      let rule_id = match diagnostic.code {
-        Some(NumberOrString::String(rule)) => rule,
-        _ => continue,
-      };
-      let ranges = error_id_to_ranges.entry(rule_id).or_insert_with(Vec::new);
-      ranges.push(diagnostic.range);
+  async fn on_code_action(&self, params: CodeActionParams) -> Option<CodeActionResponse> {
+    let text_doc = params.text_document;
+    let path = text_doc.uri.to_file_path().ok()?;
+    let diagnostics = params.context.diagnostics;
+    let error_id_to_ranges = build_error_id_to_ranges(diagnostics);
+    let mut response = CodeActionResponse::new();
+
+    let code_action = params.context.only.as_ref()?.first()?.clone();
+
+    // we only handle these code_actions
+    // 1. QuickFix
+    // 2. "source.fixAll" and "source.fixAll.ast-grep"
+    if code_action != CodeActionKind::QUICKFIX
+      && code_action != CodeActionKind::SOURCE_FIX_ALL
+      && code_action != SOURCE_FIX_ALL_AST_GREP
+    {
+      return Some(response);
     }
-    error_id_to_ranges
+
+    self
+      .client
+      .log_message(
+        MessageType::LOG,
+        format!("Running CodeAction {}", code_action.as_str()),
+      )
+      .await;
+
+    let changes = self.compute_all_fixes(text_doc, error_id_to_ranges, path);
+
+    let edit = Some(WorkspaceEdit {
+      changes,
+      document_changes: None,
+      change_annotations: None,
+    });
+    let action = CodeAction {
+      title: "Source Code fix action".to_string(),
+      command: None,
+      diagnostics: None,
+      edit,
+      disabled: None,
+      kind: Some(code_action),
+      is_preferred: Some(true),
+      data: None,
+    };
+
+    response.push(CodeActionOrCommand::from(action));
+    Some(response)
   }
 
   // TODO: support other urls besides file_scheme
@@ -424,71 +489,116 @@ impl<L: LSPLang> Backend<L> {
     let path = uri.to_file_path().ok()?;
     L::from_path(path)
   }
-}
 
-#[cfg(test)]
-mod test {
-  use super::*;
-  use ast_grep_core::language::TSLanguage;
-  use std::path::Path;
-  use tokio::io::{duplex, AsyncReadExt, AsyncWriteExt, DuplexStream};
+  async fn on_execute_command(&self, params: ExecuteCommandParams) -> Option<Value> {
+    let ExecuteCommandParams {
+      arguments,
+      command,
+      work_done_progress_params: _,
+    } = params;
 
-  #[derive(Clone, Deserialize, PartialEq, Eq)]
-  pub enum TypeScript {
-    Tsx,
-  }
-  impl Language for TypeScript {
-    fn from_path<P: AsRef<Path>>(_path: P) -> Option<Self> {
-      Some(TypeScript::Tsx)
-    }
-    fn get_ts_language(&self) -> TSLanguage {
-      tree_sitter_typescript::language_tsx().into()
+    match command.as_ref() {
+      APPLY_ALL_FIXES => {
+        self.on_apply_all_fix(command, arguments).await?;
+        None
+      }
+      _ => {
+        self
+          .client
+          .log_message(
+            MessageType::LOG,
+            format!("Unrecognized command: {}", command),
+          )
+          .await;
+        None
+      }
     }
   }
 
-  fn start_lsp() -> (DuplexStream, DuplexStream) {
-    let rc: RuleCollection<TypeScript> = RuleCollection::try_new(vec![]).unwrap();
-    let rc_result: std::result::Result<_, String> = Ok(rc);
-    let (service, socket) = LspService::build(|client| Backend::new(client, rc_result)).finish();
-    let (req_client, req_server) = duplex(1024);
-    let (resp_server, resp_client) = duplex(1024);
+  fn on_apply_all_fix_impl(&self, first: Value) -> std::result::Result<WorkspaceEdit, LspError> {
+    let text_doc: TextDocumentItem =
+      serde_json::from_value(first).map_err(LspError::JSONDecodeError)?;
+    let uri = text_doc.uri;
+    let path = uri.to_file_path().map_err(|_| LspError::InvalidFileURI)?;
+    let Some(lang) = Self::infer_lang_from_uri(&uri) else {
+      return Err(LspError::UnsupportedFileType);
+    };
 
-    // start server as concurrent task
-    tokio::spawn(Server::new(req_server, resp_server, socket).serve(service));
+    let version = text_doc.version;
+    let root = AstGrep::new(text_doc.text, lang);
+    let versioned = VersionedAst { version, root };
 
-    (req_client, resp_client)
+    let Some(diagnostics) = self.get_diagnostics(&uri, &versioned) else {
+      return Err(LspError::NoActionableFix);
+    };
+    let error_id_to_ranges = build_error_id_to_ranges(diagnostics);
+
+    let changes =
+      self.compute_all_fixes(TextDocumentIdentifier::new(uri), error_id_to_ranges, path);
+    let workspace_edit = WorkspaceEdit {
+      changes,
+      document_changes: None,
+      change_annotations: None,
+    };
+    Ok(workspace_edit)
   }
 
-  fn req(msg: &str) -> String {
-    format!("Content-Length: {}\r\n\r\n{}", msg.len(), msg)
+  async fn on_apply_all_fix(&self, command: String, arguments: Vec<Value>) -> Option<()> {
+    self
+      .client
+      .log_message(
+        MessageType::INFO,
+        format!("Running ExecuteCommand {}", command),
+      )
+      .await;
+    let first = arguments.first()?.clone();
+    let workspace_edit = match self.on_apply_all_fix_impl(first) {
+      Ok(workspace_edit) => workspace_edit,
+      Err(error) => {
+        self.report_error(error).await;
+        return None;
+      }
+    };
+    self.client.apply_edit(workspace_edit).await.ok()?;
+    None
   }
 
-  // A function that takes a byte slice as input and returns the content length as an option
-  fn resp(input: &[u8]) -> Option<&str> {
-    let input_str = std::str::from_utf8(input).ok()?;
-    let mut splits = input_str.split("\r\n\r\n");
-    let header = splits.next()?;
-    let body = splits.next()?;
-    let length_str = header.trim_start_matches("Content-Length: ");
-    let length = length_str.parse::<usize>().ok()?;
-    Some(&body[..length])
-  }
-
-  async fn test_lsp() {
-    let initialize = r#"{"jsonrpc":"2.0","method":"initialize","params":{"capabilities":{"textDocumentSync":1}},"id":1}"#;
-    let (mut req_client, mut resp_client) = start_lsp();
-    let mut buf = vec![0; 1024];
-
-    req_client
-      .write_all(req(initialize).as_bytes())
-      .await
-      .unwrap();
-    let _ = resp_client.read(&mut buf).await.unwrap();
-
-    assert!(resp(&buf).unwrap().starts_with('{'));
-  }
-  #[test]
-  fn actual_test() {
-    tokio::runtime::Runtime::new().unwrap().block_on(test_lsp());
+  async fn report_error(&self, error: LspError) {
+    match error {
+      LspError::JSONDecodeError(e) => {
+        self
+          .client
+          .log_message(
+            MessageType::ERROR,
+            format!("JSON deserialization error: {}", e),
+          )
+          .await;
+      }
+      LspError::InvalidFileURI => {
+        self
+          .client
+          .log_message(MessageType::ERROR, "Invalid URI format")
+          .await;
+      }
+      LspError::UnsupportedFileType => {
+        self
+          .client
+          .log_message(MessageType::ERROR, "Unsupported file type")
+          .await;
+      }
+      LspError::NoActionableFix => {
+        self
+          .client
+          .log_message(MessageType::LOG, "No actionable fix")
+          .await;
+      }
+    }
   }
 }
+
+enum LspError {
+  JSONDecodeError(serde_json::Error),
+  InvalidFileURI,
+  UnsupportedFileType,
+  NoActionableFix,
+}
